function [a,samp_num,n]=ALCSampling(s,d,m)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Description:
%
% This function goes with the demo codes for the following submission:
% [1] M. Boloursaz, E. S. Farahani, N. Salarieh, and F. Marvasti, Level Crossing Speech Sampling and its Sparsity Promoting Reconstruction 
%     using the Iterative Method with Adaptive Thresholding (IMAT), Submitted to IEEE Transactions on Audio, Speech and Language Processing, March 2015
%
% This function implements Adaptive Level Crossing (ALC) sampling as proposed by
% the following reference: 
%
% M. Malmirchegini, M. Kafashan, M. Ghassemian, and F. Marvasti, Non-uniform Sampling based on Adaptive Level Crossing Scheme, IET Signal Processing Journal, Accepted.
%
% Usage:
%   [a,samp_num,n]=ALCfunc(s,d,m)
% Inputs:
%   s           an input signal that we will take samples of it
%   d           distance between the two levels of ALC method
% Outputs:
%   a           output ALC sampled signal
%   samp_num    a matrix contains the numbers of samples
%   n           percent of sampling rate
% Written By: Nikan Salarieh
%             Mahdi Boloursaz Mashhadi
%
% Affiliation: 
%          Advanced Communications Research Institute (ACRI)
% Electrical Engineering Department, Sharif University of Technology
%                              Tehran, Iran
%
% For any problems, contact me at Nikan_fds@yahoo.com
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


d1=d;                                 %distance between Crossing Levels
L(1)=(s(1)+s(2)+s(3))/3;L(2)=L(1)+d1; %initial value of Crossing Levels

a=zeros(1,m);samp_num=ones(1,m);n=0;
z=zeros(m,2);
a=zeros(1,m);
k=1;
kk=1;

Lone=1;
Ltwo=1;

while k<m
    
%distances between Levels and signal
        for i=1:length(s)
            for j=1:2
            z(i,j)=s(i)-L(j);    
            end
        end
        
%find the level crossing time indexes
            for i=1:length(s)-1
         one(i)=z(i,1)*z(i+1,1);
            end
            one=one(1:length(s)-1);
            Lone=find(one<0);
            
            for i=1:length(s)-1
         two(i)=z(i,2)*z(i+1,2);
            end
            two=two(1:length(s)-1);
            Ltwo=find(two<0);
            
            
            
            if length(Ltwo)==0 & length(Lone)==0
                break;
            end
            if length(Ltwo)==1 
                Ltwo(2)=length(s);
            end
            if length(Lone)==1 
                Lone(2)=length(s);
            end
           if length(Lone)==0 & length(Ltwo)~=0
                Lone=length(s);
            end
            if length(Ltwo)==0 & length(Lone)~=0
                Ltwo=length(s);
            end
% main part 
              if Lone(1)<Ltwo(1)
% case one: the crossing levels, intersect the input
%           signal in regions with positive slopes
                if Ltwo(1)<Lone(2)
                    L(1)=L(1)+d;
                    L(2)=L(2)+d;
                    if a(k)~=s(Lone(1))
                    k=Lone(1)+m-length(s);a(k)=s(Lone(1));
                    s=s(Lone(1):length(s));
                    else
                    s=s(Lone(1)+1:length(s)); 
                    end
% case two: the slope of the input signal
%           changes from positive to negative
                elseif Ltwo(1)>Lone(2)
                    L(1)=L(1)-d;
                    L(2)=L(2)-d;
                    if a(k)~=s(Lone(1))
                        k=Lone(1)+m-length(s);   
                        a(k)=s(Lone(1));
                        if Lone(2)>=length(s)
                            Lone(2)=Lone(2)-1;
                        end
                        if s(Lone(2))>s(Lone(2)+1)
                        s=s(Lone(2):length(s));
                        else
                        s=s(Lone(1):length(s));
                        end
                    else
                        s=s(Lone(1)+1:length(s));
                    end
                 elseif Lone(2)==Ltwo(1)
                    L(1)=L(1)-d;
                    L(2)=L(2)-d;
                    if a(k)~=s(Lone(1))
                        k=Lone(1)+m-length(s);   
                        a(k)=s(Lone(1));
                        s=s(Lone(1):length(s)); 
                    else
                        s=s(Lone(1)+1:length(s));
                    end
                elseif s(1)>=s(2)
                    L(1)=L(1)-d;
                    L(2)=L(2)-d;
                end
              end
              
              
              if Lone(1)>Ltwo(1)
% case three: the crossing levels, intersect the input
%             signal in regions with negative slopes
                  if  Lone(1)<Ltwo(2)
                    L(1)=L(1)-d;
                    L(2)=L(2)-d;
                    if a(k)~=s(Ltwo(1))
                    k=Ltwo(1)+m-length(s);a(k)=s(Ltwo(1));
                    s=s(Ltwo(1):length(s));
                    else
                    s=s(Ltwo(1)+1:length(s));
                    end
% case four: the slope of the input signal
%            changes from negative to positive
                elseif  Lone(1)>Ltwo(2)
                    L(1)=L(1)+d;
                    L(2)=L(2)+d;
                    if a(k)~=s(Ltwo(1))
                        k=Ltwo(1)+m-length(s);   
                        a(k)=s(Ltwo(1));
                        if Ltwo(2)>=length(s)
                            Ltwo(2)=Ltwo(2)-1;
                        end
                        if s(Ltwo(2))<s(Ltwo(2)+1)
                        s=s(Ltwo(2):length(s));
                        else
                        s=s(Ltwo(1):length(s));
                        end
                    else
                        s=s(Ltwo(1)+1:length(s));
                    end
                    
                elseif Lone(1)==Ltwo(2)
                    L(1)=L(1)+d;
                    L(2)=L(2)+d;
                    if a(k)~=s(Ltwo(1))
                         k=Ltwo(1)+m-length(s);   
                        a(k)=s(Ltwo(1));
                        s=s(Ltwo(1):length(s));
                    else
                        s=s(Ltwo(1)+1:length(s));
                    end
                 elseif s(1)<=s(2)
                    L(1)=L(1)+d;
                    L(2)=L(2)+d;
                    
                end
              end
              
              
              
              if Lone(1)==Ltwo(1)
                  if s(Lone(1))<=s(Lone(1)+1)
                    L(1)=L(1)+d;
                    L(2)=L(2)+d;
                    if a(k)~=s(Lone(1))
                    k=Lone(1)+m-length(s);a(k)=s(Lone(1));
                    end
                    s=s(Lone(1):length(s));
                  elseif s(Lone(1))>s(Lone(1)+1)
                    L(1)=L(1)-d;
                    L(2)=L(2)-d;
                    if a(k)~=s(Ltwo(1))
                    k=Ltwo(1)+m-length(s);a(k)=s(Ltwo(1));
                    end
                    s=s(Ltwo(1):length(s));
                  end
              end
            
samp_num(kk)=k;kk=kk+1;
end
%Sampling Rate
v=0;
w=0;
for i=1:m
if a(i)==0
v=v+1;
else
    w=w+1;
end
end
n=w/(w+v)*100;

end